<?php

namespace App\Models;

use Laravel\Sanctum\HasApiTokens;
use Illuminate\Support\Facades\Hash;
use Illuminate\Notifications\Notifiable;
use Illuminate\Contracts\Auth\MustVerifyEmail;
use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Foundation\Auth\User as Authenticatable;

class User extends Authenticatable
{
    use HasApiTokens, HasFactory, Notifiable;
    use SoftDeletes;
    /**
     * The attributes that are mass assignable.
     *
     * @var array<int, string>
     */
    protected $fillable = [
        'name',
        'password',
        'username',
        'email',
        'residence_id',
        // 'shift_from',
        // 'shift_to'
        'phone',
        'role_id',
    ];
    protected $dates        = ['deleted_at'];

    /**
     * The attributes that should be hidden for serialization.
     *
     * @var array<int, string>
     */
    protected $hidden = [
        'password',
        'remember_token',
    ];

    /**
     * The attributes that should be cast.
     *
     * @var array<string, string>
     */
    protected $casts = [
        'email_verified_at' => 'datetime',
    ];

    // many to many relationship between users table and securable table
    public function securables()
    {
        return $this->belongsToMany(Securable::class,'users_securables','user_id','securable_id');
    }

    // many to many relationship between users table and permissions table
    public function permissions()
    {
        return $this->belongsToMany(Permission::class,'users_securables','user_id','permission_id');
    }

    // one to many relationship between users table and orders table
    public function order_created_by()
    {
        return $this->hasMany(Order::class,'created_by');
    }

    public function order_modified_by()
    {
        return $this->hasMany(Order::class,'modified_by');
    }

    // one to many relationship between users table and categoried table
    public function category_created_by()
    {
        return $this->hasMany(Category::class,'created_by');
    }

    public function category_modified_by()
    {
        return $this->hasMany(Category::class,'modified_by');
    }


    // public function getByEmail($email)
    // {
    //     $user = User::where('email',$email)
    //                ->first();
    //                return $user;
    // }
    // public function resetPassword($email ,$password)
    // {
    //         $user = User::where('email', $email)
    //         ->update(['password' => Hash::make($password)]);
    // }


}



