<?php

namespace App\Http\Controllers\dashboard;

use App\Http\Controllers\Controller;
use App\Imports\CategoriesImport;
use App\Models\Category;
use App\Models\Product;
use Illuminate\Http\Request;
use Maatwebsite\Excel\Facades\Excel;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;

use Flash;

class CategoryController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */

    // get all categories
    public function index()
    {
        $categories = Category::orderBy('created_at','desc')->get();
        return view('dashboard.categories.index', compact('categories'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    // get create view
    public function create()
    {
        $categories = Category::all();
        $category = new Category();
        return view('dashboard.categories.edit')->with([
            'categories'=>$categories ,
            'category' => $category,
            'status' =>'add'
        ]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
    //   dd($request->all());

        if(!isset($request->id)){
            $rules  =  array(
                'name_en' => 'string |max:191 | required',
                'name_ar' => 'string |max:191 | required',
                'code'    => 'string| unique:categories,code,NULL,id,deleted_at,NULL',
                'digital_menu' => 'string ',
            );
        }else{
            $rules  =  array(
                'name_en' => 'string |max:191 | required',
                'name_ar' => 'string |max:191 | required',
                'code' => [
                    'string',
                    Rule::unique('categories')->ignore($request->id)->where(fn ($query) => $query->whereNotNull('deleted_at')->orWhereNull('deleted_at'))
                ],
                'digital_menu' => 'string ',
            );
        }



        $validator = Validator::make($request->all(), $rules);
        if ($validator->fails()) {
            Flash::error($validator->errors());
            return back()->withErrors($validator->errors());
        }
       $data = $request->input();
        $category = null;
        if(!array_key_exists('id',$data) || !$data['id'] )
        {
            $category = new Category();
            $category->parent_id = $request->input('parent_id');
            $category->created_by =auth()->user()->id;
        }
        else{
            $category = Category::find($request->id);
            $category->modified_by =auth()->user()->id;
        }
           $category->name_en = $request->input('name_en');
            $category->name_ar = $request->input('name_ar');
            $category->code = $request->input('code');
            $category->discount = $request->input('dicount');
            $category->parent_id = array_key_exists('parent_id',$data) ? $data['parent_id'] :null;
            $category->digital_menu = $request->has('digital_menu');
            $category->pos = $request->has('pos');
            $category->save();
            return redirect(route('categories.index'));
            Flash::success('Category Created');
            return redirect(route('categories.index'));

    }


    public function getById($id)
    {
        $category = Category::find($id);
        $categories = Category::all();
        return view('dashboard.categories.edit')->with([
            'categories'=>$categories,
            'category'=> $category,
            'status'=>'show']);
    }

    public function edit($id)
    {
        $category = Category::find($id);
        $categories = Category::all();
        return view('dashboard.categories.edit')->with([
            'categories'=>$categories,
            'category'=> $category,
            'status' =>'edit'
        ]);
      }

    public function destroy($id)
    {
        $category = Category::with('sub_category')->where('id','=',$id)->first();
        if (empty($category)) {
            Flash::error('Category Not Found');
            return Response([
                'status'=>0,
                'msg'=>'Category Not Found'
            ]);
        }
        //take sub categories
        $sub_categories             = $category->sub_category;
        if(!is_null($sub_categories)){
            foreach($sub_categories as $sub_category){
                // get syb category products
                $sub_category_products = $sub_category->products;
                // loop on sub category products
                foreach($sub_category_products as $sub_category_product){
                    $product_orders = $sub_category_product->orders;
                    // dd($product_orders);
                    if(count($product_orders)>0){
                        return Response([
                            'status'=>0,
                            'msg'=>"Can't delete category because exist product belongs to order"
                        ]);
                    }else{
                        // get sub category product items
                        $sub_category_product->items()->detach();
                        Product::where('id','=',$sub_category_product->id)->delete();
                    }
                }
                $sub_category->delete();
            }
        }
        // get all category products
        $category_products = $category->products;
        foreach($category_products as $category_product){
            $cat_product_orders = $category_product->orders;
            if(count($cat_product_orders)>0){
                return Response([
                    'status'=>0,
                    'msg'=>"Can't delete category because exist product belongs to order"
                ]);
            }else{
                $category_product->items()->detach();
                $category_product->delete();
            }

        }
        $category->delete();
          return Response([
              'status'=>1,
              'msg'=>'Category deleted'
          ]);
      }

      public function test($data)
     {

        $rules  =  array(
            'name_en' => 'string |max:191 | required',
            'name_ar' => 'string |max:191 | required',
            'code'    => 'string| unique:categories,code,NULL,id,deleted_at,NULL',
            'digital_menu' => 'string ',
        );

          $validator = Validator::make($data, $rules);

          if ($validator->fails()) {

            Flash::error($validator->errors());
            return redirect()->route('uploadFiles_page')->with('Import Faild',$validator->errors());

            return back()->withErrors($validator->errors());
        }
     }
}





