<?php

namespace App\Http\Controllers\dashboard;

use App\Models\Log;
use App\Models\User;
use App\Models\Order;
use App\Models\Invoice;
use App\Models\Product;
use App\Models\Category;
use App\Models\Settings;
use App\Models\Inventory;
use App\Models\Order_Item;
use App\Models\Product_Item;
use Illuminate\Http\Request;
use Salla\ZATCA\Tags\Seller;
use Salla\ZATCA\GenerateQrCode;
// use App\Models\order_invoice_items;
use Salla\ZATCA\Tags\TaxNumber;
use Salla\ZATCA\Tags\InvoiceDate;
use App\Models\product_ingredient;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Auth;
use Salla\ZATCA\Tags\InvoiceTaxAmount;
use Salla\ZATCA\Tags\InvoiceTotalAmount;

class Order_InvoicesController extends Controller
{
    // get all orders
    public function orders(){
        $orders = Order::orderBy('created_at','desc')->get();

        $log    = new Log();
        $log->username  = Auth::user()->username;
        $log->action    = 'show all orders';
        $log->save();
        // dd($orders);
        return view('dashboard.orders.index')->with([
            'orders'    => $orders
        ]);
    }

    // add new order
    public function add_order(){
        $order_id   = Order::orderBy('id','DESC')->first();
        if(!is_null($order_id)){
            $last_order_id = $order_id->id + 1;
        }else{
            $last_order_id = 1;
        }

        $order_number_len = strlen((string)$last_order_id);
        switch($order_number_len){
            case('1'):
                $order_number = '00000'.$last_order_id;
                break;
            case('2'):
                $order_number = '0000'.$last_order_id;
                break;
            case('3'):
                $order_number = '000'.$last_order_id;
                break;
            case('4'):
                $order_number = '00'.$last_order_id;
                break;
            case('6'):
                $order_number = '0'.$last_order_id;
                break;
            default:
                $order_number = $last_order_id;
                break;
        }

        $categories     = Category::where('pos','1')->get();
        $settings       = Settings::first();
        $open_orders    = Order::where('order_status','1')->get();

        $log    = new Log();
        $log->username  = Auth::user()->username;
        $log->action    = 'open pos page';
        $log->save();

        // dd($settings);
        return view('dashboard.orders.add')->with([
            'categories'        => $categories,
            'settings'          => $settings,
            'last_order_id'     => $order_number,
            'open_orders'       => $open_orders
        ]);
    }

    // show order
    public function show_order($id){
        $order          = Order::where('id',$id)->with('products')->first();
        $created_by     = User::find($order->created_by);
        // dd($order);
        $modified_by    = User::find($order->modified_by);
        $settings       = Settings::first();

        $log    = new Log();
        $log->username  = Auth::user()->username;
        $log->action    = 'show order its num '.$id;
        $log->save();
        return view('dashboard.orders.show')->with([
            'order'         => $order,
            'created_by'    => $created_by,
            'modified_by'   => $modified_by,
            'settings'      => $settings
        ]);
    }

    //edit order
    public function edit_order($id){
        $order          = Order::where('id',$id)->with('products')->first();
        $categories     = Category::where('pos','1')->get();
        $settings       = Settings::first();
        $open_orders    = Order::where('order_status','1')->get();

        $order_id   = Order::orderBy('id','DESC')->first();

        if(!is_null($order_id)){
            $last_order_id = $order_id->id + 1;
        }else{
            $last_order_id = 1;
        }
        // dd($last_order_id);
        $order_number_len = strlen((string)$last_order_id);
        switch($order_number_len){
            case('1'):
                $order_number = '00000'.$last_order_id;
                break;
            case('2'):
                $order_number = '0000'.$last_order_id;
                break;
            case('3'):
                $order_number = '000'.$last_order_id;
                break;
            case('4'):
                $order_number = '00'.$last_order_id;
                break;
            case('6'):
                $order_number = '0'.$last_order_id;
                break;
            default:
                $order_number = $last_order_id;
                break;
        }

        $log    = new Log();
        $log->username  = Auth::user()->username;
        $log->action    = 'open order its num '.$id . ' to edit it';
        $log->save();
        // dd($order_number);
        return view('dashboard.orders.add')->with([
            'categories'        => $categories,
            'settings'          => $settings,
            'last_num_order'    => $last_order_id,
            'order'             => $order,
            'open_orders'       => $open_orders
        ]);
    }

    // insert_update_order
    public function insert_update_order(Request $request){
        // dd($request->all());
        $order_id           = $request->order_id;
        $products           = explode(',',$request->products);
        $quantity           = $request->quantity;
        $product_dis        = $request->product_dis;
        $sub_total          = $request->sub_total;
        $sub_total_a_dis    = $request->sub_total_a_dis;
        $tax                = $request->tax;
        $tax_a_dis          = $request->tax_a_dis;
        $discount           = $request->discount;
        $total              = $request->total;
        $order_type         = $request->order_type;
        $customer           = $request->customer;
        $waiter_name        = $request->waiter_name;
        $table_num          = $request->table_num;
        $payment_type       = $request->payment_type;

        $order              = new Order();
        if(!is_null($order_id)){
            $order                  = Order::where('id','=',$order_id)->first();
            $order->modified_by     = auth()->user()->id;

            $log    = new Log();
            $log->username  = Auth::user()->username;
            $log->action    = 'update order its num '.$order_id;
            $log->save();
            // $order->modified_by     =1;
        }else{
            $log            = new Log();
            $log->username  = Auth::user()->username;
            $log->action    = 'submit new order ';
            $log->save();
        }

        $order->order_type      = $order_type;
        $order->table_num       = $table_num;
        $order->customer_name   = $customer;
        $order->waiter_name     = $waiter_name;
        $order->payment_type    = $payment_type;
        $order->created_date    = now();
        $order->created_by      = auth()->user()->id;
        // $order->created_by      = 1;
        $order->sub_total       = $sub_total;
        $order->sub_total_a_dis = $sub_total_a_dis;
        $order->tax             = $tax;
        $order->tax_a_dis       = $tax_a_dis;
        $order->discount        = $discount;
        $order->total           = $total;
        $order->order_status    = 1;
        $prev_items             = [];
        $prev_qty               = [];
        $order->save();

        if(!is_null($order_id)){
            // $prev_order     = order_invoice_items::where('order_id',$order_id)->get();
            $prev_order     = Order_Item::where('order_id',$order_id)->get();
            foreach($prev_order as $prev_o){
                $prev_items[] = $prev_o->product_id;
                $prev_qty  [$prev_o->product_id][] = $prev_o->quantity;
            }
            // $del_items      = order_invoice_items::where('order_id',$order_id)->delete();
            $del_items      = Order_Item::where('order_id',$order_id)->delete();
        }

        $settings = Settings::first();
        for($i=0 ; $i < count($products) ; $i++){
            $product   = Product::where('id',$products[$i])->first();
            // taxable_amount = sub_total (غير شامل الضريبة) without vat
            // total_amount   = total (شامل الضريبة) with vat

            if($settings->with_tax){
                $product_with_qty           = $product->selling_price * $quantity[$i];
                $product_price              = $product_with_qty * (100 / (100+$settings->tax));
                $product_vat                = $product_with_qty - $product_price   ;
                $product_total_amount       = $product_price + $product_vat;
            }else{
                $product_price              = $product->selling_price * $quantity[$i];
                $product_vat                = $product_price * $settings->tax / 100;
                $product_total_amount       = $product_price + $product_vat ;
                // dd($product_total_amount);
            }

            // $order_item                     = new order_invoice_items();
            $order_item                     = new Order_Item();
            $order_item->order_id           = $order->id;
            $order_item->product_id         = $products[$i];
            $order_item->quantity           = $quantity[$i];
            $order_item->product_dis        = $product_dis[$i];
            $order_item->taxable_amount     = $product_price;
            $order_item->total_amount       = $product_total_amount;
            $order_item->tax_amount         = $product_vat;
            $order_item->save();
        }

        // decrise from inventory
        if(is_null($order_id)){
            for($i=0 ; $i < count($products) ; $i++){
                $product        = Product::where('id',$products[$i])->with('items')->first();
                $items          = $product->items;
                // $items          = Product_Item::where('product_id',$products[$i])
                //                     ->join('items','items.id','item_id')->toSql();
                if(count($items)>0){
                    foreach($items as $it){
                        $new_inventory          = $it->pivot->mesurment * $quantity[$i];
                        $inventory              = Inventory::where('item_id',$it->id)->first();
                        if(!is_null($inventory)){
                            $inventory->quantity    = $inventory->quantity - $new_inventory;
                            $inventory->save();
                        }
                    }
                }
            }
        }else{
            for($i=0;$i<count($prev_items);$i++){
                $product        = Product::where('id',$prev_items[$i])->with('items')->first();
                $items          = $product->items;
                if(count($items)>0){
                    foreach($items as $it){
                        $new_inventory          = $it->pivot->mesurment * $prev_qty[$prev_items[$i]][0];
                        $inventory              = Inventory::where('item_id',$it->id)->first();
                        if(!is_null($inventory)){
                            $inventory->quantity    = $inventory->quantity + $new_inventory;
                            $inventory->save();
                        }
                    }
                }
            }
            for($i=0 ; $i < count($products) ; $i++){
                $product        = Product::where('id',$products[$i])->with('items')->first();
                $items          = $product->items;
                if(count($items)>0){
                    foreach($items as $it){
                        $new_inventory          = $it->pivot->mesurment * $quantity[$i];
                        $inventory              = Inventory::where('item_id',$it->id)->first();
                        if(!is_null($inventory)){
                            $inventory->quantity    = $inventory->quantity - $new_inventory;
                            $inventory->save();
                        }
                    }
                }
            }
        }



        // get last order num
        $order_last_id   = Order::orderBy('id','DESC')->first();
        if(!is_null($order_last_id)){
            $last_order_id = $order_last_id->id + 1;
        }else{
            $last_order_id = 1;
        }

        $order_number_len = strlen((string)$last_order_id);
        switch($order_number_len){
            case('1'):
                $order_number = '00000'.$last_order_id;
                break;
            case('2'):
                $order_number = '0000'.$last_order_id;
                break;
            case('3'):
                $order_number = '000'.$last_order_id;
                break;
            case('4'):
                $order_number = '00'.$last_order_id;
                break;
            case('6'):
                $order_number = '0'.$last_order_id;
                break;
            default:
                $order_number = $last_order_id;
                break;
        }

        return Response([
            'order'             => $order,
            'last_order_id'     => $last_order_id
        ]);
    }

    // delete order
    public function del_order(Request $request){
        $order_id   = $request->id;
        $order      = Order::find($order_id);
        $order->delete();
        $order->save();

        $log    = new Log();
        $log->username  = Auth::user()->username;
        $log->action    = 'delete order its num '.$order_id ;
        $log->save();
        if ($order->trashed()) {
            return Response(['status'=>'success']);
        }else{
            return Response(['status'=>'error']);
        }
    }

    // cancel order
    public function cancel_order(Request $request){
        $order_id               = $request->id;
        $order                  = Order::find($order_id);
        $order->order_status    = 2;
        $order->save();

        // $prev_order     = order_invoice_items::where('order_id',$order_id)->get();
        $prev_order     = Order_Item::where('order_id',$order_id)->get();
        foreach($prev_order as $prev_o){
            // $prev_items[] = $prev_o->product_id;
            // $prev_qty  [$prev_o->product_id][] = $prev_o->quantity;
            $product        = Product::where('id',$prev_o->product_id)->with('items')->first();
            $items          = $product->items;
            foreach($items as $it){
                $new_inventory          = $it->pivot->mesurment * $prev_o->quantity;
                $inventory              = Inventory::where('item_id',$it->id)->first();
                $inventory->quantity    = $inventory->quantity + $new_inventory;
                $inventory->save();
            }
        }

        $log    = new Log();
        $log->username  = Auth::user()->username;
        $log->action    = 'cancel order its num '.$order_id ;
        $log->save();
        // for($i=0;$i<count($prev_items);$i++){

        // }
        return Response($order);
    }

    // close order
    public function close_order(Request $request){
        // dd($request->payment_type);
        $order_id           = $request->order_id;
        $products           = explode(',',$request->products);
        // $quantity           = $request->quantity;
        $sub_total          = $request->sub_total;
        $sub_total_a_dis    = $request->sub_total_a_dis;
        $tax                = $request->tax;
        $tax_a_dis          = $request->tax_a_dis;
        $discount           = $request->discount;
        $total              = $request->total;
        $order_type         = $request->order_type;
        $customer           = $request->customer;
        $waiter_name        = $request->waiter_name;
        $table_num          = $request->table_num;
        $payment_type       = $request->payment_type;
        $prev_items             = [];
        $prev_qty               = [];
        $order              = Order::where('id',$order_id)->first();
        if($order_id  == 0 || is_null($order)){
            $order                  = new Order();
            $log    = new Log();
            $log->username  = Auth::user()->username;
            $log->action    = 'close order not submitted before ';
            $log->save();
        }else{
            $order                  = Order::where('id',$order_id)->first();
            $order->modified_by     = auth()->user()->id;

            $log    = new Log();
            $log->username  = Auth::user()->username;
            $log->action    = 'close order its num '.$order_id . ' had submitted before';
            $log->save();
            // $order->modified_by     = 1;
        }

        $order->order_type      = $order_type;
        $order->table_num       = $table_num;
        $order->customer_name   = $customer;
        $order->waiter_name     = $waiter_name;
        $order->payment_type    = $payment_type;
        $order->created_date    = now();
        $order->created_by      = auth()->user()->id;
        // $order->created_by      = 1;
        $order->sub_total       = $sub_total;
        $order->sub_total_a_dis = $sub_total_a_dis;
        $order->tax             = $tax;
        $order->tax_a_dis       = $tax_a_dis;
        $order->discount        = $discount;
        $order->total           = $total;
        $order->order_status    = 3;
        $order->save();

        if($order_id  != 0){
            // $prev_order     = order_invoice_items::where('order_id',$order_id)->get();
            $prev_order     = Order_Item::where('order_id',$order_id)->get();
            foreach($prev_order as $prev_o){
                $prev_items[] = $prev_o->product_id;
                $prev_qty  [$prev_o->product_id][] = $prev_o->quantity;
            }
            // $del_items      = order_invoice_items::where('order_id',$order_id)->delete();
            $del_items      = Order_Item::where('order_id',$order_id)->delete();
        }

        $settings = Settings::first();
        for($i=0 ; $i < count($products) ; $i++){
            $product   = Product::where('id',$products[$i])->first();
            // taxable_amount = sub_total (غشر شامل الضريبة)
            // total_amount   = total (شامل الضريبة)

            // if($settings->with_tax){
            //     $product_price              =($product->selling_price * (100 / (100+$settings->tax)))*$quantity[$i];
            //     $product_vat                = $product->selling_price - $product_price   ;
            //     $product_total_amount       = $product_price + $product_vat;
            // }else{
            //     $product_price              = $product->selling_price * $quantity[$i];
            //     $vat                        = $product_price * $settings->tax / 100;
            //     $product_total_amount       = $product_price + $vat ;
            //     // dd($product_total_amount);
            // }

            if($settings->with_tax){
                $product_with_qty           = $product->selling_price * $request->quantity[$i];
                $product_price              = $product_with_qty * (100 / (100+$settings->tax));
                $product_vat                = $product_with_qty - $product_price   ;
                $product_total_amount       = $product_price + $product_vat;
                // $product_price              =($product->selling_price * (100 / (100+$settings->tax)))*$request->quantity[$i];
                // $product_vat                = $product->selling_price - $product_price   ;
                // $product_total_amount       = $product_price + $product_vat;
            }else{

                $product_price              = $product->selling_price * $request->quantity[$i];
                $product_vat                = $product_price * $settings->tax / 100;
                $product_total_amount       = $product_price + $product_vat ;
            }

            // $order_item                     = new order_invoice_items();
            $order_item                     = new Order_Item();
            $order_item->order_id           = $order->id;
            $order_item->product_id         = $products[$i];
            $order_item->quantity           = $request->quantity[$i];
            $order_item->taxable_amount     = $product_price;
            $order_item->total_amount       = $product_total_amount;
            $order_item->tax_amount         = $product_vat;
            // $order_item->taxable_amount     = $product_taxable_amount;
            // $order_item->total_amount       = $product_total_amount;
            // $order_item->tax_amount         = $product_tax_amount;
            $order_item->save();
        }

        // decrise from inventory
        if($order_id  == 0){
            for($i=0 ; $i < count($products) ; $i++){
                $product        = Product::where('id',$products[$i])->with('items')->first();
                $items          = $product->items;
                foreach($items as $it){
                    $new_inventory          = $it->pivot->mesurment * $request->quantity[$i];
                    $inventory              = Inventory::where('item_id',$it->id)->first();
                    if(!is_null($inventory)){
                        $inventory->quantity    = $inventory->quantity - $new_inventory;
                        $inventory->save();
                    }
                }
            }
        }else{
            for($i=0;$i<count($prev_items);$i++){
                $product        = Product::where('id',$prev_items[$i])->with('items')->first();
                $items          = $product->items;
                foreach($items as $it){
                    $new_inventory          = $it->pivot->mesurment * $prev_qty[$prev_items[$i]][0];
                    $inventory              = Inventory::where('item_id',$it->id)->first();
                    if(!is_null($inventory)){
                        $inventory->quantity    = $inventory->quantity + $new_inventory;
                        $inventory->save();
                    }
                }
            }
            for($i=0 ; $i < count($products) ; $i++){
                $product        = Product::where('id',$products[$i])->with('items')->first();
                $items          = $product->items;
                foreach($items as $it){
                    $new_inventory          = $it->pivot->mesurment * $request->quantity[$i];
                    $inventory              = Inventory::where('item_id',$it->id)->first();
                    if(!is_null($inventory)){
                        $inventory->quantity    = $inventory->quantity - $new_inventory;
                        $inventory->save();
                    }
                }
            }
        }

        $invoice                = new Invoice();
        $invoice->order_id      = $order->id ;
        $invoice->save();

        // dd($invoice);
        return Response($invoice);
        //$this->print_order($order->id);
    }

    // print order
    public function print_order($id){
        $order          = Order::where('id',$id)->first();
        $product_items  = $order->products()->get();
        $settings       = Settings::first();

        $log    = new Log();
        $log->username  = Auth::user()->username;
        $log->action    = 'print order its num '.$id;
        $log->save();
        return view('dashboard.orders.print')->with([
            'order'         => $order,
            'settings'      => $settings,
            'products'      => $product_items,
        ]);
    }

    // get order
    public function get_order(Request $request){
        $order          = Order::where('id',$request->id)->first();
        $product_items  = $order->products()->get();
        $order_last_id   = Order::orderBy('id','DESC')->first();
        if(!is_null($order_last_id)){
            $last_order_id = $order_last_id->id + 1;
        }else{
            $last_order_id = 1;
        }

        $order_number_len = strlen((string)$last_order_id);
        switch($order_number_len){
            case('1'):
                $order_number = '00000'.$last_order_id;
                break;
            case('2'):
                $order_number = '0000'.$last_order_id;
                break;
            case('3'):
                $order_number = '000'.$last_order_id;
                break;
            case('4'):
                $order_number = '00'.$last_order_id;
                break;
            case('6'):
                $order_number = '0'.$last_order_id;
                break;
            default:
                $order_number = $last_order_id;
                break;
        }

        return Response([
            'order'             =>$order,
            'products'          =>$product_items,
            'last_order_id'     => $order_number,

        ]);
    }

    // get category products
    public function get_products(Request $request){
        $cat_id         = $request->cat_id;
        if(isset($request->barcode)){
            $products       = Product::where('barcode','=',$request->barcode)->first();
            $category       = Category::where('id',$products->category_id)->first();
        }else{
            $category       = Category::where('id',$cat_id)->first();
            $products       = $category->products()->get();
        }

        return Response([
            'products'      => $products,
            'category'      => $category
        ]);
    }


    // --------------------------------------------------------------------
    // get all invoices
    public function all_invoices(){
        $invoices   = Invoice::with('order')->orderBy('created_at','desc')->get();

        $log    = new Log();
        $log->username  = Auth::user()->username;
        $log->action    = 'show all invoices';
        $log->save();
        // dd($invoices[0]->order);
        return view('dashboard.invoices.index')->with([
            'invoices'  => $invoices
        ]);
    }

    // show Invoice
    public function show_invoice($id){
        $invoice        = Invoice::where('id',$id)->with('order')->first();
        $product_items  = $invoice->order->products()->get();
        $created_by     = User::find($invoice->order->created_by);
        $modified_by    = User::find($invoice->order->modified_by);
        $settings       = Settings::first();

        $log    = new Log();
        $log->username  = Auth::user()->username;
        $log->action    = 'show invoice its num '.$id;
        $log->save();
        return view('dashboard.invoices.show')->with([
            'invoice'       => $invoice,
            'settings'      => $settings,
            'products'      => $product_items,
            'modified_by'   => $modified_by,
            'created_by'    => $created_by
        ]);
    }

    // print invoice
    public function print_invoice($id){
        $invoice        = Invoice::where('id',$id)->with('order')->first();
        $product_items  = $invoice->order->products()->get();
        $settings       = Settings::first();

        $log    = new Log();
        $log->username  = Auth::user()->username;
        $log->action    = 'print invoice  its num '.$id;
        $log->save();
        $generatedString = GenerateQrCode::fromArray([
            new Seller($settings->coffee_name), // seller name
            new TaxNumber($settings->vat_number), // seller vat_number
            new InvoiceDate($invoice->created_at), // invoice date as Zulu ISO8601 @see https://en.wikipedia.org/wiki/ISO_8601
            new InvoiceTotalAmount($invoice->order->total), // invoice total amount with vat
            new InvoiceTaxAmount($invoice->order->tax) // invoice tax amount
            // TODO :: Support others tags
        ])->toBase64();

        $displayQRCodeAsBase64 = GenerateQrCode::fromArray([
            new Seller($settings->coffee_name), // seller name
            new TaxNumber($settings->vat_number), // seller vat_number
            new InvoiceDate($invoice->created_at), // invoice date as Zulu ISO8601 @see https://en.wikipedia.org/wiki/ISO_8601
            new InvoiceTotalAmount($invoice->order->total), // invoice total amount with vat
            new InvoiceTaxAmount($invoice->order->tax) // invoice tax amount
            // TODO :: Support others tags
        ])->render();
        // dd($displayQRCodeAsBase64);
        return view('dashboard.invoices.print')->with([
            'invoice'       => $invoice,
            'settings'      => $settings,
            'products'      => $product_items,
            'QR'            => $displayQRCodeAsBase64
        ]);
    }

    // get all opened orders
    public function opening_orders($id){
        if($id == 0){
            $open_orders    = Order::where('order_status','1')->get();
        }else{
            $open_orders    = Order::where('order_status','1')->where('id','!=',$id)->get();
        }
        return Response(['opening_orders'=>$open_orders]);
    }
}
