<?php

namespace App\Http\Controllers\dashboard;

use App\Http\Controllers\Controller;
use App\Imports\ProductsImport;
use App\Models\Category;

use App\Models\Product;
use App\Models\Item;

use Illuminate\Http\Request;
use Maatwebsite\Excel\Facades\Excel;
use Flash;
use Illuminate\Support\Facades\Validator;

class ProductController extends Controller
{

    public function index()
    {
        $products = Product::with('category')->orderBy('created_at','desc')->with(['category','items'])->get();
        $categories = Category::all();
        return view('dashboard.products.index')->with([
            'products'      => $products,
            'categories'    => $categories
        ]);
    }

    /**
     * Show the form for creating a new product.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $categories = Category::all();
        $items = Item::all();
        return view('dashboard.products.add')->with('categories', $categories)
         ->with('items',$items)
        ->with('status','add');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //1- check user permission
          //code to add check user permission
        //2 -validation
        $rule  =  array(
            'name_en' => 'string|required|max:255',
            'name_ar' => 'string|required|max:255',
            'code' => 'string',
            'cost_price' => 'required',
            'selling_price' => 'required',
            'image' => 'nullable|image|mimes:jpeg,png,jpg,gif,svg|max:2048',
            'category_id' => 'required',
        ) ;
        $validator = Validator::make($request->all(),$rule);
        if($validator->fails()) {
            Flash::error($validator->errors());
            return back()->withErrors($validator->errors());
          }
             $data = $request->input();
             $product = null;
            if(!array_key_exists('id',$data) || !$data['id'] )
              {
                 $product = new Product();
              }
             /* else
              {
                  $product = Product::where('id','=',$id)->first();
                //  $product->items()->detach();
              }*/

            $product->name_en = $request->input('name_en');
            $product->name_ar = $request->input('name_ar');
            $product->code = $request->input('code');
            $product->cost_price = $request->input('cost_price');
            $product->selling_price = $request->input('selling_price');
            $product->discount = $request->input('discount');
            $pos=$request->input('pos');
            if($pos =='on'){
                $product->pos =1;
            }else {
                $product->pos =0;
            }
            $digital_menu=$request->input('digital_menu');
            if($digital_menu =='on'){
                $product->digital_menu =1;
            }else {
                $product->digital_menu =0;
            }
            $product->barcode = $request->input('barcode');
            $product->category_id = $request->input('category_id');
            $image = $request->file('image');
            if (isset($image)) {
                $imageName = time() . '.' . $image->getClientOriginalExtension();
                $image->move('images', $imageName);
                $product->image = $imageName;
            }
            $product->save();
            // for item exits
            if ($request->detail_id ) {
                $items = $request->detail_id;
                foreach ($items as $item) {
                   //  $item
                   $item_p = Item::where('id','=',$request->item_en[$item])->first();
                    if($item_p)
                    {
                      //ite is already created
                      $product ->items()->attach($item_p->id,['mesurment'=>$request->measurement[$item]]);
                    }
                }
            }

            if ($request->detail_id_1 ) {
                $items_1 = $request->detail_id_1;
                foreach ($items_1 as $item_1) {
                    $item = Item::create([
                        'name_en' => $request->item_name_en[$item_1],
                        'name_ar' => $request->item_name_ar[$item_1],
                        'unit' => $request->item_unit[$item_1]
                    ]);
                 //   $item->name_en = $request->item_name_en[$item_1];
                 //   $item->name_ar = $request->item_name_ar[$item_1];
                //    $item->unit = $request->item_unit[$item_1];
                    //attach item to product
                    $product->items()->attach($item->id,['mesurment'=>$request->measurement_item[$item_1]]);
                }
            }
            if(!array_key_exists('id',$data) || !$data['id'] )
            Flash::success('Product Created');
            else
            Flash::success('Product Updated');
            return redirect(route('products.index'));
    }

   public function update($product,Request $request)
   {
    {
        //1- check user permission
          //code to add check user permission
        //2 -validation
        $rule  =  array(
            'name_en' => 'string|required|max:255',
            'name_ar' => 'string|required|max:255',
            'code' => 'string',
            'cost_price' => 'required',
            'selling_price' => 'required',
            'image' => 'nullable|image|mimes:jpeg,png,jpg,gif,svg|max:2048',
            'category_id' => 'required',
        ) ;
        $validator = Validator::make($request->all(),$rule);
        if($validator->fails()) {
            Flash::error($validator->errors());
            return back()->withErrors($validator->errors());
          }
            $data = $request->input();
            $product = Product::where('id','=',$product)->first();
            $product->name_en = $request->input('name_en');
            $product->name_ar = $request->input('name_ar');
            $product->code = $request->input('code');
            $product->cost_price = $request->input('cost_price');
            $product->selling_price = $request->input('selling_price');
            $product->discount = $request->input('discount');
            $pos=$request->input('pos');
            if($pos =='on'){
                $product->pos =1;
            }else {
                $product->pos =0;
            }
            $digital_menu=$request->input('digital_menu');
            if($digital_menu =='on'){
                $product->digital_menu =1;
            }else {
                $product->digital_menu =0;
            }
            $product->barcode = $request->input('barcode');
            $product->category_id = $request->input('category_id');
            $image = $request->file('image');
            if (isset($image)) {
                $imageName = time() . '.' . $image->getClientOriginalExtension();
                $image->move('images', $imageName);
                $product->image = $imageName;
            }
            $product->save();
            // for item exits
            if ($request->detail_id ) {
                $items = $request->detail_id;
                foreach ($items as $item) {
                   //  $item
                   $item_p = Item::where('id','=',$request->item_en[$item])->first();
                    if($item_p)
                    {
                      //ite is already created
                      $product ->items()->attach($item_p->id,['mesurment'=>$request->measurement[$item]]);
                    }
                }
            }

            if ($request->detail_id_1 ) {
                $items_1 = $request->detail_id_1;
                foreach ($items_1 as $item_1) {
                    $item = Item::create([
                        'name_en' => $request->item_name_en[$item_1],
                        'name_ar' => $request->item_name_ar[$item_1],
                        'unit' => $request->item_unit[$item_1]
                    ]);
                 //   $item->name_en = $request->item_name_en[$item_1];
                 //   $item->name_ar = $request->item_name_ar[$item_1];
                //    $item->unit = $request->item_unit[$item_1];
                    //attach item to product
                    $product->items()->attach($item->id,['mesurment'=>$request->measurement_item[$item_1]]);
                }
            }
            Flash::success('Product Updated');
            return redirect(route('products.index'));
    }
   }
    public function show($id)
    {
        $product = Product::where('id' , $id)->with(['items','category'])->first();

        if (empty($product)) {
            Flash::error('Product Not Found');

            return redirect(route('products.index'));
        }

        return view('dashboard.products.show',compact('product'));
            //->with('categories', $categories);

    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $product = Product::where('id','=',$id)->with(['category','items'])->first();
        $categories = Category::all();
        $items = $items = Item::all();//$product ->items;
        // $ingredients =product_ingredient::where('product_id',$id)
        //                 ->join('ingredients','ingredients.id','product_ingredient.ingredient_id')->get();



        if (empty($product)) {
            Flash::error('Product Not Found');

            return redirect(route('products.index'));
        }

        return view('dashboard.products.edit')->with('product', $product)
            ->with('categories',$categories)
            ->with('items',$items);

    }



    public function deleteProduct($id)
    {
         $product = Product::find($id);
         $items = $product->items;
         $product->items()->detach();
        if (empty($product)) {
            Flash::error('Product Not Found');
            return redirect(route('products.index'));
        }
        $product->delete();

        //Flash::success('Product deleted');
        return redirect(route('products.index'));;
    }

    public function delete_i($id)
    {

        $item = Item::find($id);

       $items->delete();
       Flash::success('Item deleted');
           return  true;
    }

    public function destroy($id)
    {
          $product = Product::where('id','=',$id)->first();
          if (empty($product)) {
              Flash::error('Product Not Found');
              return Response([
                  'status'=>0,
                  'msg'=>'Product Not Found'
              ]);
              // return redirect(route('categories.index'));
          }else{
            if(count($product->orders)>0){
                return Response([
                    'status'=>0,
                    'msg'=>"Can't delete product because its belong to order"
                ]);
            }else{
                $product->items()->detach();
                $product->delete();
            }
          }


          Flash::success('Product deleted');

          return Response([
              'status'=>1,
              'msg'=>'Product deleted'
          ]);
    }
}
