<?php

namespace App\Http\Controllers\dashboard;
use Flash;
use PdfReport;
use App\Models\Role;
use App\Models\Order;
use App\Models\Invoice;
use App\Models\Product;
use App\Models\Category;
use App\Models\Purchase;
use App\Models\Supplier;
use App\Models\Order_Item;
use Illuminate\Http\Request;
use App\Exports\ReportExport;
use Illuminate\Support\Facades\DB;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\View;
use Maatwebsite\Excel\Facades\Excel;
use Illuminate\Database\Eloquent\Model;

class ReportController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */

    public function index(Request $request)
    {
        $relevant_classes =["Category", "Product","Order","Purchase","Inventory"];
        if (request()->ajax()) {
            if(!in_array($request->model, $relevant_classes ))
            {
                throw new \Exception('Invalid model');
            }
            $modelName = '\\App\\Models\\'.$request->model;
            $model     = new $modelName;
            // dd(class_basename($model),in_array(class_basename($model), $relevant_classes));
            if (class_basename($model) == 'Category') {
                if (!empty($request->from_date)) {
                    $data = DB::table('categories as c')->select('name_en')
                    ->leftJoin('products as p','c.id','=','p.category_id')
                    ->leftJoin('inventories as i','p.name_en','=','i.item_en')
                    ->select('p.id','p.name_en','p.selling_price','i.quantity')
                    ->whereBetween(DB::raw('date(c.created_at)'), [$request->from_date, $request->to_date])
                    ->get();
                    // $data = $model::whereBetween('created_at', [$request->from_date, $request->to_date])->get();
                }
                else {
                    $data = DB::table('categories as c')->get();
                }
                return dataTables()->of($data)->removeColumn('image','updated_at','created_at','dicount','dicount','pos','digital_menu','code','parent_cat')->make(true);
            }
        }
        return response()->view('dashboard.reports.index');
    }


    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */


    public function displayReport(Request $request)
    {
        $fromDate = $request->input('from_date');
        $toDate = $request->input('to_date');
        $sortBy = $request->input('sort_by');

        $title = 'Registered Stands Report'; // Report title

        $meta = [ // For displaying filters description on header
            'Registered on' => $fromDate . ' To ' . $toDate,
            'Sort By' => $sortBy
        ];

        $queryBuilder = Category::select(['name', 'price','status', 'created_at']) // Do some querying..
        ;
        //    ->whereBetween('created_at', [$fromDate, $toDate])
        //    ->orderBy($sortBy)
        $columns = [ // Set Column to be displayed
            'Name' => 'name',
            'Registered At'=>'created_at', // if no column_name specified, this will automatically seach for snake_case of column name (will be registered_at) column from query result
            'Price' => 'price',
            'Status' =>'status',
        ];

        // Generate Report with flexibility to manipulate column class even manipulate column value (using Carbon, etc).
        return PdfReport::of($title, $meta, $queryBuilder, $columns)
            ->editColumn('Created At', [ // Change column class or manipulate its data for displaying to report
                'displayAs' => function($result) {
                    return $result->created_at->format('d M Y');
                },
                'class' => 'left'
            ])
            ->editColumns(['price', 'Status'], [ // Mass edit column
                 'class' => 'right bold'
            ])
            ->showTotal([ // Used to sum all value on specified column on the last table (except using groupBy method). 'point' is a type for displaying total with a thousand separator
                'price' => 'point' // if you want to show dollar sign ($) then use 'Total Balance' => '$'
            ])
            ->limit(20) // Limit record to be showed
            ->stream(); // other available method: store('path/to/file.pdf') to save to disk, download('filename') to download pdf / make() that will producing DomPDF / SnappyPdf instance so you could do any other DomPDF / snappyPdf method such as stream() or download()
    }

    public function getReports(Request $request)
    {
        $relevant_classes =["Category", "Product","Order","Purchase","Inventory"];
        // $modelName = '\\App\\Models\\'.$request->model;
        // $model     = new $modelName;
        $result = array();
        $dates = $this->dateDiffInDays($request->from_date, $request->to_date);

        switch(class_basename($request->model)) {
            case('Product'):

                // get order ids in invoices according to date filter
                $invoice_ids = Invoice::
                    whereBetween(DB::raw('date(created_at)'), [$request->from_date, $request->to_date])
                    // where('created_at', '>', $request->from_date)
                    //                    ->where('created_at', '<', $request->to_date)
                                        ->pluck('order_id')->toArray();
                                        //new
                $products = Order_Item::whereIn('order_id',$invoice_ids)
                                ->select(
                                DB::raw("COUNT(id) as count"),
                                DB::raw("SUM(quantity) as Quantity"),
                                DB::raw("SUM(total_amount) as Total"),
                                DB::raw("SUM(taxable_amount) as TotalWithoutTax"),
                                DB::raw("SUM(tax_amount) as TaxAmount"),
                                DB::raw('product_id')
                                )
                                ->with('product')
                                ->groupBy('product_id')
                                ->get();


                if(count($products)>0){
                    foreach ($products as $product_order) {

                        $quanaties = $product_order->Quantity;
                        $tax= round($product_order->TaxAmount,2);
                        $total_without_vat =round($product_order->TotalWithoutTax,2);
                        $total_with_vat =round($product_order->Total ,2);
                            $name_ar = $product_order->product->name_ar;
                            $name_en =$product_order ->product ->name_en;
                            $price = $product_order ->product ->selling_price;
                        array_push($result , ["Arabic Name" =>$name_ar, "English Name" =>$name_en, "price" =>$price ,"Quantity" =>$quanaties ,"Total without Vat"=>round($total_without_vat ,2),"vat" =>round($tax,2) , "Total" =>round($total_with_vat,2)  ,"Number Of Invoices"=>$product_order->count]);
                    }
                }else{
                    array_push($result , ["Arabic Name" =>'', "English Name" =>'', "price" =>'' ,"Quantity" =>'' ,"Total without Vat"=>'',"vat" =>'' , "Total" =>''  ,"Number Of Invoices"=>'']);
                }

                $data1 = dataTables()->of($result)->removeColumn('image','updated_at','created_at','dicount','dicount','pos','digital_menu','code','parent_cat')->make(true);
                return Response([
                    'data1' => $data1
                ]);
                // return dataTables()->of($result)->removeColumn('image','updated_at','created_at','dicount','dicount','pos','digital_menu','code','parent_cat')->make(true);
                break;

            case('Category'):

                $invoice_ids = Invoice::
                    whereBetween(DB::raw('date(created_at)'), [$request->from_date, $request->to_date])
                ->pluck('order_id')->toArray();

                $categries = Category::whereHas('products.orders', function($query) use($invoice_ids)
                    { $query -> whereIn('order_id', $invoice_ids);})
                    ->with('products.orders')
                    ->get();

                if(count($categries)>0){
                    foreach($categries as $categry){
                        $categry_id =$categry->id;
                        $name_en  =$categry->name_en;
                        $name_ar =$categry ->name_ar;
                        $quanitity = 0 ;
                        $tax =0;
                        $total_without_vat=0;
                        $total =0;
                        $count =0;
                        $products = Order_Item::whereIn('order_id',$invoice_ids)
                                            ->whereHas('product' , function($q) use($categry_id){
                                                    $q->where('category_id',$categry_id);
                                                })
                                            ->with(['product'])
                                            ->select(
                                                    DB::raw("COUNT(id) as count"),
                                                    DB::raw("SUM(quantity) as Quantity"),
                                                    DB::raw("SUM(total_amount) as Total"),
                                                    DB::raw("SUM(taxable_amount) as TotalWithoutTax"),
                                                    DB::raw("SUM(tax_amount) as TaxAmoumt"),
                                                    DB::raw('product_id')
                                                )
                                            ->groupBy('product_id')
                                            ->get();

                        foreach($products as $product_order)
                        {
                            $quanitity += $product_order->Quantity;
                            $tax += $product_order->TaxAmoumt;
                            $total_without_vat += $product_order->TotalWithoutTax;
                            $total += $product_order->Total;
                            $count+=$product_order->count;
                        }
                        array_push($result , ["Arabic Name" =>$name_ar, "English Name" =>$name_en,  "Quantity" =>$quanitity ,"Total without Vat"=>round($total_without_vat,2) ,"vat" =>round($tax,2) , "Total" =>round($total,2) ,"Number of Invoices"=>$count]);
                    }
                }else{
                    array_push($result , ["Arabic Name" =>'', "English Name" =>'',  "Quantity" =>'' ,"Total without Vat"=>'' ,"vat" =>'' , "Total" =>'' ,"Number of Invoices"=>'']);
                }
                $data1 = dataTables()->of($result)->removeColumn('image','updated_at','created_at','dicount','dicount','pos','digital_menu','code','parent_cat')->make(true);
                return Response([
                    'data1' => $data1
                ]);
                // return dataTables()->of($result)->removeColumn('image','updated_at','created_at','dicount','dicount','pos','digital_menu','code','parent_cat')->make(true);

                break;
            case('Order'):

                $invoice_ids = Invoice::all()
                // whereBetween(DB::raw('date(created_at)'), [$request->from_date, $request->to_date])
                ->pluck('order_id') ->toArray();
                // res array for calculate all orders
                $res = array();
                $orders = Order::whereBetween(DB::raw('date(updated_at)'), [$request->from_date, $request->to_date])

                ->select(
                    DB::raw("COUNT(id) as OrderNumbers"),
                    DB::raw("SUM(total) as Total"),
                    DB::raw("SUM(sub_total) as TotalWithoutTax"),
                    DB::raw("SUM(tax) as TotalTax")
                )->get();
                $total = $orders[0]['Total'];
                $totalWithoutVat =round( $orders[0]['TotalWithoutTax'],2);
                $totalTax = round($orders[0]['TotalTax'],2);
                $object = new \stdClass();
                $object ->OrderNumbers = round($orders[0]['OrderNumbers'],2);
                $object->Total = round($total,2);
                $object->TotalWithoutVat=round($totalWithoutVat,2);
                $object ->totalTax = round($totalTax,2);


                $open_orderd = Order::whereBetween(DB::raw('date(updated_at)'), [$request->from_date, $request->to_date])

                ->where('order_status',1)
                ->select(
                DB::raw("COUNT(id) as openOrderd")
                ) ->first();
                // $cancel_orderd = Order::whereIn('id', $invoice_ids)
                $cancel_orderd = Order::whereBetween(DB::raw('date(updated_at)'), [$request->from_date, $request->to_date])
                ->where('order_status',2)
                ->select(
                DB::raw("COUNT(id) as canceleOrders")
                ) ->first();
                // $closed_orderd = Order::whereIn('id', $invoice_ids)
                $closed_orderd = Order::whereBetween(DB::raw('date(updated_at)'), [$request->from_date, $request->to_date])
                ->where('order_status',3)
                ->select(
                DB::raw("COUNT(id) as closed")
                ) ->first();
                $object->openOrders = $open_orderd['openOrderd'];
                $object->cancelOrders = $cancel_orderd['canceleOrders'];
                $object->closedOrders = $closed_orderd['closed'];
                array_push($res,["Order Numbers" =>$object ->OrderNumbers ,"Total" =>round($total,2) ,"Total Without Vat" =>round($totalWithoutVat,2) ,"Total Tax"=>round($totalTax,2),"Open Oreders" =>$open_orderd['openOrderd']  ,"Cancle Orders" =>$cancel_orderd['canceleOrders'] ,"Close Orders"=>$closed_orderd['closed'] ]);


                // return each order in array
                $result =  Order::whereBetween(DB::raw('date(updated_at)'), [$request->from_date, $request->to_date])

                ->select(
                'order_status as Status','total as Total' , 'sub_total as TotalWithoutTax' , 'tax as TotalTax'

                )->get()->map(function ($result){
                    switch($result->Status){
                        case '1':
                            $result->Status = 'opened';
                            break;
                        case '2':
                            $result->Status = 'canceled';
                            break;
                        case '3':
                            $result->Status = 'closed';
                            break;
                        default:
                            break;
                    };
                });


                    $orders_details = Order::whereBetween(DB::raw('date(updated_at)'), [$request->from_date, $request->to_date])

                    ->select(
                    DB::raw("DATE(created_at) as date"),
                        DB::raw("COUNT(id) as OrderNumbers"),
                        DB::raw("SUM(total) as Total"),
                        DB::raw("SUM(sub_total) as TotalWithoutTax"),
                        DB::raw("SUM(tax) as TotalTax")
                    )
                    ->groupBy('date')
                    ->get();
                    //
                    $detail_res = array();
                    if(count($orders_details)>0){
                        foreach($orders_details as $object)
                        {
                            array_push($detail_res,["Date" =>$object->date  ,
                                                        "Order Numbers" =>$object ->OrderNumbers ,
                                                        "Total" =>round($object ->Total,2) ,
                                                        "Total Without Vat" =>round($object ->TotalWithoutTax,2) ,
                                                        "Total Tax"=>round($object ->TotalTax,2)
                                                        ]);

                        }
                    }else{
                        array_push($detail_res,["Date" =>'',"Order Numbers" =>'',"Total" =>'',"Total Without Vat" =>'',"Total Tax"=>'']);
                    }
                    //  dd($res);
                    // dd($detail_res);
                    // res is for first table
                    //detail_res is array according to dates

                    $data1 = dataTables()->of($res)->removeColumn('image','updated_at','created_at','dicount','dicount','pos','digital_menu','code','parent_cat')->make(true);
                    $data2 = dataTables()->of($detail_res)->make(true);
                    return Response([
                        'data1' => $data1,
                        'data2' => $data2
                    ]);



                break;
            case('Purchase'):
                $res = array();

                $toal_purchase =  Purchase::  whereBetween(DB::raw('date(created_at)'), [$request->from_date, $request->to_date])
                ->select(
                    DB::raw("COUNT(id) as Purchasesnumbers"),
                    DB::raw("SUM(paid_amount) as paid_amount"),
                    DB::raw("SUM(total_amount) as total_amount"),

                    )
                    ->first();

                    $res = array();
                    array_push($res ,["Purchaese Numbers" =>$toal_purchase['Purchasesnumbers']
                                    ,"Total Paid Amount" =>round($toal_purchase['paid_amount'],2),
                                    "Total amount" =>round($toal_purchase['total_amount'],2)

                    ]);
                    //detail purchase
                        ///purchases details according to dates
                $details =  Purchase::  whereBetween(DB::raw('date(created_at)'), [$request->from_date, $request->to_date])
                ->select(
                DB::raw("COUNT(id) as Purchasesnumbers"),
                    DB::raw("DATE(created_at) as date"),
                    DB::raw("SUM(paid_amount) as paid_amount"),
                    DB::raw("SUM(total_amount) as total_amount"),

                    )
                        ->groupBy('date')
                        ->get();
                $detail_res = array();
                if(count($details)>0){
                    foreach($details as $p)
                    {

                        array_push($detail_res ,["Date" => $p['date'],
                                                    "Purchaese Numbers" =>$p['Purchasesnumbers']
                                                    ,"Total Paid Amount" =>round($p['paid_amount'],2),
                                                    "Total amount" =>round($p['total_amount'],2)

                                                    ]);
                    }
                }else{
                    array_push($detail_res ,["Date" =>'',"Purchaese Numbers" =>'',"Total Paid Amount" =>'',"Total amount" =>'']);
                }


                $data1 = dataTables()->of($res)->removeColumn('image','updated_at','created_at','dicount','dicount','pos','digital_menu','code','parent_cat')->make(true);
                $data2 = dataTables()->of($detail_res)->make(true);
                return Response([
                    'data1' => $data1,
                    'data2' => $data2
                ]);
                //   return dataTables()->of($result)->make(true);
                break;
            case ('Invoices') :
                    $res = array();
                    $order_ids =  Invoice::whereBetween(DB::raw('date(updated_at)'), [$request->from_date, $request->to_date])
                        ->pluck('order_id')->toArray();
                    $res = array();
                    $orders = Order::whereIn('id' ,$order_ids)
                        ->select(
                        DB::raw("COUNT(id) as OrderNumbers"),
                        DB::raw("SUM(total) as Total"),
                        DB::raw("SUM(sub_total) as TotalWithoutTax"),
                        DB::raw("SUM(sub_total_a_dis) as TotalAfterDisc"),
                        DB::raw("SUM(tax_a_dis) as TaxAfterDisc"),
                        DB::raw("SUM(discount) as discount"),
                        DB::raw("SUM(tax) as TotalTax")
                        )
                        ->first();
                        // $res paraneyer is for result
                        array_push($res,["Invoice Numbers" =>$orders['OrderNumbers'] ,"Total Without Vat" =>round($orders['TotalWithoutTax'],2) ,"Tax" =>round($orders['TotalTax'],2) ,"Discount"  =>round($orders['discount'],2),"Total" =>round($orders['Total'],2) ]);
                    //get details
                    $orders_by_dates = Order::whereIn('id' ,$order_ids)
                    ->select(

                    DB::raw("DATE(created_at) as date"),
                    DB::raw("COUNT(id) as OrderNumbers"),
                    DB::raw("SUM(total) as Total"),
                    DB::raw("SUM(sub_total) as TotalWithoutTax"),
                    DB::raw("SUM(sub_total_a_dis) as TotalAfterDisc"),
                    DB::raw("SUM(tax_a_dis) as TaxAfterDisc"),
                    DB::raw("SUM(discount) as discount"),
                    DB::raw("SUM(tax) as TotalTax")
                        )
                    ->groupBy('date')
                    ->get();

                    $collection = collect($orders_by_dates);
                        $detail_res = array();

                    if(count($orders_by_dates)>0){
                        foreach($orders_by_dates as $object)
                        {
                            array_push($detail_res,["Date" =>$object->date  ,
                                                    "Invoice Numbers" =>$object ->OrderNumbers ,
                                                    "Total" =>round($object ->Total,2) ,
                                                    "Total Without Vat" =>round($object ->TotalWithoutTax,2) ,
                                                    "Total Tax"=>round($object ->TotalTax,2)
                                                    ]);

                        }
                    }else{
                        array_push($detail_res,["Date" =>'',"Invoice Numbers" =>'',"Total" =>'',"Total Without Vat" =>'',"Total Tax"=>'']);
                    }
                    // dd($detail_res);

                        //detail result array according to days
                    // $detail_res = $collection->groupBy('date')->map(function ($row) {
                    //     $detail_res = array();
                    //     foreach($row as $orders)
                    //     {
                    //         array_push($detail_res ,[
                    //                 "Date" =>$orders['date'] ,
                    //                 "Order Numbers" =>$orders['OrderNumbers'] ,
                    //                 "Total Without Vat" =>round($orders['TotalWithoutTax'],2) ,
                    //                 "Tax" =>round($orders['TotalTax'],2) ,"Discount"  =>round($orders['discount'],2),
                    //                 "Total" =>round($orders['Total'],2)]);

                    //      }
                    //     return  $detail_res;

                    // });

                    $data1 = dataTables()->of($res)->removeColumn('image','updated_at','created_at','dicount','dicount','pos','digital_menu','code','parent_cat')->make(true);
                    $data2 = dataTables()->of($detail_res)->make(true);
                    return Response([
                        'data1' => $data1,
                        'data2' => $data2
                    ]);


                    // return dataTables()->of($res)->removeColumn('image','updated_at','created_at','dicount','dicount','pos','digital_menu','code','parent_cat')->make(true);
                    break;
                default:


        }
    }
    function dateDiffInDays($date1, $date2)
    {
        $diff = strtotime($date2) - strtotime($date1);
        return abs(round($diff / 86400));
    }


    public function print_report(Request $request){
        if($request->type == 1){
            $view = View::make('dashboard.reports.print')->with(['content'   => $request->table]);
            return $view->render();
        }else{
           return  Excel::download(new ReportExport($request->table1,$request->table2,$request->date,$request->report_type), 'Report '.$request->date.'.xlsx');
        }
    }
}
