<?php

namespace App\Http\Controllers\dashboard;

use App\Http\Controllers\Controller;
use App\Models\Supplier;
use Illuminate\Http\Request;
use Flash;
use Illuminate\Support\Facades\Validator;

class SupplierController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $suppliers= Supplier::all();
        return view('dashboard.suppliers.index',compact('suppliers'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('dashboard.suppliers.add');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
        $rule  =  array(
            'name' => 'string |max:191 | required',
            'vat_number' => 'string',
            'email' =>'email|nullable',
            'contact_person_email'=>'email|nullable',
            'contact_person_phone'=>'numeric|nullable'
        ) ;

        $validator = Validator::make($request->all(),$rule);
        if($validator->fails()) {
            Flash::error($validator->errors());
            return back()->withErrors($validator->errors());
        } else {
            $supplier = new Supplier();
            $supplier->name = $request->input('name');
            $supplier->email = $request->input('email');
            $supplier->registeration_number = $request->input('registeration_number');
            // $supplier->payment_method = $request->input('payment_method');
            $supplier->address = $request->input('address');
            $supplier->city = $request->input('city');
            $supplier->country = $request->input('country');
            $supplier->vat_number = $request->input('vat_number');
            $supplier->contact_person = $request->input('contact_person');
            $supplier->contact_person_phone = $request->input('contact_person_phone');
            $supplier->contact_person_email = $request->input('contact_person_email');

            $supplier->save();
            Flash::success('Supplier Created');
            return redirect(route('suppliers.index'));
    }
}

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
        $supplier=Supplier::find($id);
        return view('dashboard.suppliers.show',compact('supplier'));
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        //
        $supplier=Supplier::find($id);
        return view('dashboard.suppliers.edit',compact('supplier'));

    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
        $supplier=Supplier::find($id);
        $supplier->name = $request->input('name');
        $supplier->email = $request->input('email');
        $supplier->registeration_number = $request->input('registeration_number');
        // $supplier->payment_method = $request->input('payment_method');
        $supplier->address = $request->input('address');
        $supplier->city = $request->input('city');
        $supplier->country = $request->input('country');
        $supplier->vat_number = $request->input('vat_number');
        $supplier->contact_person = $request->input('contact_person');
        $supplier->contact_person_phone = $request->input('contact_person_phone');
        $supplier->contact_person_email = $request->input('contact_person_email');
        $supplier->save();
        return redirect(route('suppliers.index'));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
        $supplier=Supplier::find($id);

        if (empty($supplier)) {
            Flash::error('Supplier Not Found');

            return redirect(route('suppliers.index'));
        }

        $supplier->delete();

        Flash::success('Supplier deleted');

        return redirect(route('suppliers.index'));
    }
}
