<?php

namespace App\Http\Controllers\dashboard;

use Flash;
use App\Models\Role;
use App\Models\User;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;


class UserController extends Controller
{

    // function to get all users
    public function all_users()
    {
        $users= User::with('role')->get();
        return view('dashboard.users.index',compact('users'));
    }

    // function to Show the form for creating a new user.
    public function add_user()
    {
        $roles          = Role::all();
        $check_owner    = User::with('role')->where('role_id','2')->first();
        // dd(is_null($check_owner));
        return view('dashboard.users.add')->with([
            'roles'          => $roles,
            'status'        => 'add',
            'check_owner'   => $check_owner,
        ]);
    }

    // function to insert update user
    public function insert_update_user(Request $request){
       // dd($request->url());
        $language = app()->getLocale();
        if($language == 'en'){
            $name_max_error             = 'Full Name should be max lenght 191 character';
            $name_required_error        = 'Full Name is required';
            $name_string_error          = 'Full Name should be string';
            $username_required_error    = 'User Name is required';
            $username_string_error      = 'User Name should be string';
            $password_required_error    = 'User Password is required';
            $password_string_error      = 'User Password should be string';
            $residence_id_required_error= 'Residence ID is required';
            $residence_id_string_error  = 'Residence ID should be string';
            $role_required_error        = 'Role is required';
            $role_integer_error         = 'Role should be integer';
            $email_required_error       = 'User Email is required';
            $email_string_error         = 'User Email should be string';
            $email_string_unique        = 'User Email should be unique';
            $phone_required_error       = 'User Phone is required';
            $phone_string_error         = 'User Phone should be string';
            $phone_string_unique        = 'User Phone should be unique';
        }else{
            $name_max_error             = 'الاسم الكامل يجب أن يكون طوله على الأكثر 19 محرف';
            $name_required_error        = 'الاسم الكامل مطلوب';
            $name_string_error          = 'الاسم الكامل يجب أن يكون نص';
            $username_required_error    = 'اسم المستخدم مطلوب';
            $username_string_error      = 'اسم المستخدم يجب أن يكون نص';
            $password_required_error    = 'كلمة المرور مطلوبة';
            $password_string_error      = 'كلمة المرور يجب أن تحتوي على محارف و أرقام';
            $residence_id_required_error= 'رقم المسكن مطلوب';
            $residence_id_string_error  = 'رقم المسكن يجب أن يكون نص';
            $role_required_error        = 'اختيار الدور مطلوب';
            $role_integer_error         = 'الدور يجب أن يكون رقم صحيح' ;
            $email_required_error       = 'عنوان البريد الالكتروني مطلوب';
            $email_string_error         = 'عنوان البريد الالكتروني يجب أن يكون نص';
            $email_string_unique        = 'عنوان البريد الالكتروني مستخدم من قبل مستخدم آخر';
            $phone_required_error       = 'رقم هاتف المستخدم مطلوب';
            $phone_string_error         = 'رقم هاتف المستخدم يجب أن يكون نص';
            $phone_string_unique        = 'رقم هاتف المستخدم مُستَخدمْ من قبل مُستخدِم آخر';

        }

        if(!isset($request->user_id)){
            $request->validate([
                'name'          => 'string |max:191 | required',
                'username'      =>'required|string|unique:users,username,NULL,id,deleted_at,NULL',

                // 'email'         => 'string | required | unique:users',
                'email'         => 'required|string|unique:users,email,NULL,id,deleted_at,NULL',

                'password'      => 'string | required',
                'residence_id'  => 'string | required ',
                // 'phone'         => 'string | required | unique:users',
                'phone'         => 'required|string|unique:users,password,NULL,id,deleted_at,NULL',

                'role'          => 'integer | required',
            ],[
                'name.required'                 =>$name_required_error,
                'name.string'                   =>$name_string_error,
                'name.max'                      =>$name_max_error,
                'username.required'             =>$username_required_error,
                'username.string'               =>$username_string_error,
                'email.required'                =>$email_required_error,
                'email.string'                  =>$email_string_error,
                'email.unique'                  =>$email_string_unique,
                'password.string'               =>$password_string_error,
                'password.required'             =>$password_required_error,
                'residence_id.string'           =>$residence_id_string_error,
                'residence_id.required'         =>$residence_id_required_error,
                'phone.string'                  =>$phone_string_error,
                'phone.unique'                  =>$phone_string_unique,
                'phone.required'                =>$phone_required_error,
                'role.integer'                  =>$role_integer_error,
                'role.required'                 =>$role_required_error,
            ]);
            $rules  =  array(
                'name_en' => 'string |max:191 | required',
                'name_ar' => 'string |max:191 | required',
                'code'    => 'string| unique:categories,code,NULL,id,deleted_at,NULL',
                'digital_menu' => 'string ',
            );
        }else{
            $request->validate([
                'name'          => 'string |max:191 | required',
                'username'      => [
                    'required',
                    'string',
                    Rule::unique('users','username')->ignore($request->user_id)->where(fn ($query) => $query->whereNotNull('deleted_at')->orWhereNull('deleted_at'))
                ],
                // 'email'         => 'string | required | unique:users',
                'email'         => [
                    'required',
                    'string',
                    Rule::unique('users','email')->ignore($request->user_id)->where(fn ($query) => $query->whereNotNull('deleted_at')->orWhereNull('deleted_at'))
                ],
                'password'      => 'string | required',
                'residence_id'  => 'string | required ',
                // 'phone'         => 'string | required | unique:users',
                'phone'         => [
                    'required',
                    'string',
                    Rule::unique('users','phone')->ignore($request->user_id)->where(fn ($query) => $query->whereNotNull('deleted_at')->orWhereNull('deleted_at'))
                ],
                'role'          => 'integer | required',
            ],[
                'name.required'                 =>$name_required_error,
                'name.string'                   =>$name_string_error,
                'name.max'                      =>$name_max_error,
                'username.required'             =>$username_required_error,
                'username.string'               =>$username_string_error,
                'email.required'                =>$email_required_error,
                'email.string'                  =>$email_string_error,
                'email.unique'                  =>$email_string_unique,
                'password.string'               =>$password_string_error,
                'password.required'             =>$password_required_error,
                'residence_id.string'           =>$residence_id_string_error,
                'residence_id.required'         =>$residence_id_required_error,
                'phone.string'                  =>$phone_string_error,
                'phone.unique'                  =>$phone_string_unique,
                'phone.required'                =>$phone_required_error,
                'role.integer'                  =>$role_integer_error,
                'role.required'                 =>$role_required_error,
            ]);
        }
            $request->validate([
                'name'          => 'string |max:191 | required',
                'username'      => [
                    'required',
                    'string',
                    Rule::unique('users','username')->ignore($request->user_id)->where(fn ($query) => $query->whereNotNull('deleted_at')->orWhereNull('deleted_at'))
                ],
                // 'email'         => 'string | required | unique:users',
                'email'         => [
                    'required',
                    'string',
                    Rule::unique('users','email')->ignore($request->user_id)->where(fn ($query) => $query->whereNotNull('deleted_at')->orWhereNull('deleted_at'))
                ],
                'password'      => 'string | required',
                'residence_id'  => 'string | required ',
                // 'phone'         => 'string | required | unique:users',
                'phone'         => [
                    'required',
                    'string',
                    Rule::unique('users','phone')->ignore($request->user_id)->where(fn ($query) => $query->whereNotNull('deleted_at')->orWhereNull('deleted_at'))
                ],
                'role'          => 'integer | required',
            ],[
                'name.required'                 =>$name_required_error,
                'name.string'                   =>$name_string_error,
                'name.max'                      =>$name_max_error,
                'username.required'             =>$username_required_error,
                'username.string'               =>$username_string_error,
                'email.required'                =>$email_required_error,
                'email.string'                  =>$email_string_error,
                'email.unique'                  =>$email_string_unique,
                'password.string'               =>$password_string_error,
                'password.required'             =>$password_required_error,
                'residence_id.string'           =>$residence_id_string_error,
                'residence_id.required'         =>$residence_id_required_error,
                'phone.string'                  =>$phone_string_error,
                'phone.unique'                  =>$phone_string_unique,
                'phone.required'                =>$phone_required_error,
                'role.integer'                  =>$role_integer_error,
                'role.required'                 =>$role_required_error,
            ]);

        $user = new User();


        if(isset($request->user_id)){
            $user = User::where('id','=',$request->user_id)->first();
        }

        if($request->conf_type){
            if($request->password != $request->confirm_password){
                return back()->withErrors('Confirm password should by matched password');
            }
        }
        $user->name         = $request->name;
        $user->username     = $request->username;
        $user->email        = $request->email;
        if(isset($request->user_id)){
            if($request->conf_type){
                $user->password = Hash::make($request->password);
            }else{
                $user->password = $request->password;
            }
        }else{
            $user->password     = Hash::make($request->password);
        }
        $user->residence_id = $request->residence_id;
        $user->phone        = $request->phone;
        $user->role_id      = $request->role;
        $user->shift_from   = $request->shift_from;
        $user->shift_to     = $request->shift_to;
        $user->save();
        if(isset($request->user_id)){
            Flash::success('User Updated');
            return redirect()->route('profile',['id'=>$request->user_id]);
        }else{
            Flash::success('User added Successfully');
            return redirect(route('all_users'));
            // return redirect()->back();
        }
        // if(isset($user->user_id)){
        //     return redirect(route('all_users'));
        // }else{
        //     return redirect()->back();
        // }
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        //
        $rule  =  array(
            'name' => 'string |max:191 | required',
            'username' => 'string',
            'email' => 'string',
            'password' => 'string',
            'residence_id' => 'string ',
            'phone' => 'string ',
            'role'=> 'string ',
            // 'shift_from' =>'string ',
            // 'shift_to'=>'string ',
        ) ;

        $validator = Validator::make($request->all(),$rule);
        if($validator->fails()) {
            Flash::error($validator->errors());
            return back()->withErrors($validator->errors());
        } else {
            $user = new User();
            $user->name = $request->input('name');
            $user->username = $request->input('username');
            $user->email = $request->input('email');
            $user->password = Hash::make($request->input('password'));
            $user->residence_id = $request->input('residence_id');
            $user->phone = $request->input('phone');
            $user->role = $request->input('role');
            $user->shift_from = $request->input('shift_from');
            $user->shift_to = $request->input('shift_to');
            $user->save();
            Flash::success('User Created');
            return redirect(route('users.index'));
        }
    }
    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show_user($id)
    {
        //
        $user = User::where('id','=',$id)->first();
        $roles = Role::all();
        return view('dashboard.users.add')->with([
            'user'   => $user,
            'roles'  => $roles,
            'status' => 'show',
        ]);

    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit_user($id)
    {
        //
        $user = User::where('id','=',$id)->first();
        $roles = Role::all();
        $check_owner    = User::with('role')->where('role_id','2')->first();
        return view('dashboard.users.add')->with([
            'user'  => $user,
            'roles' => $roles,
            'status'=> 'edit',
            'check_owner'=> $check_owner
        ]);

    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    // public function update(Request $request, $id)
    // {

    //     //
    //     $rule  =  array(
    //         'name' => 'string |max:191 | required',
    //         'username' => 'string',
    //         'email' => 'string',
    //         'password' => 'string',
    //         'residence_id' => 'string ',
    //         'phone' => 'string ',
    //         'role'=> 'string ',
    //         // 'shift_from' =>'string ',
    //         // 'shift_to'=>'string ',
    //     ) ;

    //     $validator = Validator::make($request->all(),$rule);
    //     if($validator->fails()) {
    //         Flash::error($validator->errors());
    //         return back()->withErrors($validator->errors());
    //     } else {

    //         if($request->conf_type){
    //             if($request->password != $request->confirm_password){
    //                 return back()->withErrors('Confirm password should by matched password');
    //             }
    //         }

    //         $user = User::find($id);
    //         $user->name = $request->input('name');
    //         $user->username = $request->input('username');
    //         $user->email = $request->input('email');
    //         if($request->conf_type){
    //             $user->password = Hash::make($request->password);
    //         }else{
    //             $user->password = $request->password;
    //         }
    //         $user->residence_id = $request->input('residence_id');
    //         $user->phone = $request->input('phone');
    //         $user->role = $request->input('role');
    //         $user->shift_from = $request->input('shift_from');
    //         $user->shift_to = $request->input('shift_to');
    //         $user->save();
    //         Flash::success('User Updated');
    //         return redirect(route('users.index'));
    //     }
    // }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $user = User::where('id','=',$id)->first();
        if (empty($user)) {
            return Response([
                'status'=>0,
                'msg'=>'User Not Found'
            ]);
        }
        $user->delete();
        return Response([
            'status'=>1,
            'msg'=>'User deleted'
        ]);
    }
}
